@@ -81,6 +81,13 @@ uint BackgroundPortal::NotifyBackground(const QDBusObjectPath &handle,
     qCDebug(XdgDesktopPortalKdeBackground) << "    app_id: " << app_id;
     qCDebug(XdgDesktopPortalKdeBackground) << "    name: " << name;
 
+    // If KWayland::Client::PlasmaWindowManagement hasn't been created, we would be notified about every
+    // application, which is not what we want. This will be mostly happening on X11 session.
+    if (!WaylandIntegration::plasmaWindowManagement()) {
+        results.insert(QStringLiteral("result"), static_cast<uint>(BackgroundPortal::Ignore));
+        return 0;
+    }
+
     KNotification *notify = new KNotification(QStringLiteral("notification"), KNotification::Persistent | KNotification::DefaultEvent, this);
     notify->setTitle(i18n("Background activity"));
     notify->setText(i18n("%1 is running in the background.", app_id));
@@ -106,8 +113,6 @@ uint BackgroundPortal::NotifyBackground(const QDBusObjectPath &handle,
     QDBusMessage message = q_ptr->message();
 
     message.setDelayedReply(true);
-    reply = message.createReply();
-    QDBusConnection::sessionBus().send(reply);
 
     connect(notify, QOverload<uint>::of(&KNotification::activated), this, [=] (uint action) {
         if (action != 1) {
@@ -131,7 +136,7 @@ uint BackgroundPortal::NotifyBackground(const QDBusObjectPath &handle,
         }
 
         const QVariantMap map = { {QStringLiteral("result"), static_cast<uint>(result)} };
-        QDBusMessage reply = message.createReply({0, map});
+        QDBusMessage reply = message.createReply({static_cast<uint>(0), map});
         if (!QDBusConnection::sessionBus().send(reply)) {
             qCWarning(XdgDesktopPortalKdeBackground) << "Failed to send response";
         }
@@ -143,7 +148,7 @@ uint BackgroundPortal::NotifyBackground(const QDBusObjectPath &handle,
 
         QVariantMap map;
         map.insert(QStringLiteral("result"), static_cast<uint>(BackgroundPortal::Ignore));
-        QDBusMessage reply = message.createReply({0, map});
+        QDBusMessage reply = message.createReply({static_cast<uint>(0), map});
         if (!QDBusConnection::sessionBus().send(reply)) {
             qCWarning(XdgDesktopPortalKdeBackground) << "Failed to send response";
         }
