@@ -27,6 +27,9 @@
 
 #include <QLoggingCategory>
 
+#include <KNotification>
+#include <KLocalizedString>
+
 Q_LOGGING_CATEGORY(XdgDesktopPortalKdeScreenCastStream, "xdp-kde-screencast-stream")
 
 class PwFraction {
@@ -277,7 +280,7 @@ void ScreenCastStream::onStreamFormatChanged(void *data, const struct spa_pod *f
                                                       SPA_PARAM_BUFFERS_buffers, SPA_POD_CHOICE_RANGE_Int(16, 2, 16),
                                                       SPA_PARAM_BUFFERS_blocks, SPA_POD_Int (1),
                                                       SPA_PARAM_BUFFERS_size, SPA_POD_Int(size),
-                                                      SPA_PARAM_BUFFERS_stride, SPA_POD_Int(stride),
+                                                      SPA_PARAM_BUFFERS_stride, SPA_POD_CHOICE_RANGE_Int(stride, stride, INT32_MAX),
                                                       SPA_PARAM_BUFFERS_align, SPA_POD_Int(16));
     pw_stream_update_params(pw->pwStream, params, 1);
 #else
@@ -365,17 +368,27 @@ void ScreenCastStream::init()
 {
     pw_init(nullptr, nullptr);
 
+    const auto emitFailureNotification = [](const QString &body) {
+        KNotification *notification = new KNotification(QStringLiteral("screencastfailure"), KNotification::CloseOnTimeout);
+        notification->setTitle(i18n("Failed to start screencasting"));
+        notification->setText(body);
+        notification->setIconName(QStringLiteral("dialog-error"));
+        notification->sendEvent();
+    };
+
 #if PW_CHECK_VERSION(0, 2, 90)
     pwMainLoop = pw_thread_loop_new("pipewire-main-loop", nullptr);
     pwContext = pw_context_new(pw_thread_loop_get_loop(pwMainLoop), nullptr, 0);
     if (!pwContext) {
         qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to create PipeWire context";
+        emitFailureNotification(i18n("Failed to create PipeWire context"));
         return;
     }
 
     pwCore = pw_context_connect(pwContext, nullptr, 0);
     if (!pwCore) {
         qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to connect PipeWire context";
+        emitFailureNotification(i18n("Failed to connect PipeWire context"));
         return;
     }
 
@@ -384,6 +397,7 @@ void ScreenCastStream::init()
     pwStream = createStream();
     if (!pwStream) {
         qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to create PipeWire stream";
+        emitFailureNotification(i18n("Failed to create PipeWire stream"));
         return;
     }
 #else
@@ -402,6 +416,7 @@ void ScreenCastStream::init()
 
     if (pw_thread_loop_start(pwMainLoop) < 0) {
         qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to start main PipeWire loop";
+        emitFailureNotification(i18n("Failed to start main PipeWire loop"));
         return;
     }
 }
@@ -513,15 +528,27 @@ bool ScreenCastStream::recordFrame(gbm_bo *bo, quint32 width, quint32 height, qu
 
     if (!(data = (uint8_t *) spa_buffer->datas[0].data)) {
         qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to record frame: invalid buffer data";
+        pw_stream_queue_buffer(pwStream, buffer);
         return false;
     }
 
-    const quint32 destStride = SPA_ROUND_UP_N(videoFormat.size.width * BITS_PER_PIXEL, 4);
-    const quint32 destSize = BITS_PER_PIXEL * width * height * sizeof(uint8_t);
-    const quint32 srcSize = spa_buffer->datas[0].maxsize;
+    quint32 streamStride;
+    const quint32 minStride = SPA_ROUND_UP_N(videoFormat.size.width * BITS_PER_PIXEL, 4);
+    const quint32 minSrcSize = height * minStride;
 
-    if (destSize != srcSize || stride != destStride) {
-        qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to record frame: different stride";
+    const quint32 srcSize = height * stride;
+    const quint32 destSize = spa_buffer->datas[0].maxsize;
+
+    // If we can fit source into the pipewire buffer, we can use stride we got from gbm_bo as the client
+    // should be able to handle it
+    if (srcSize <= destSize) {
+        streamStride = stride;
+    // Fallback to fixed minimum stride, which should be (width * bpp)
+    } else if (minSrcSize == destSize) {
+        streamStride = minStride;
+    } else {
+        qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to record frame: got buffer with higher stride than we can handle";
+        pw_stream_queue_buffer(pwStream, buffer);
         return false;
     }
 
@@ -533,6 +560,7 @@ bool ScreenCastStream::recordFrame(gbm_bo *bo, quint32 width, quint32 height, qu
 
     if (image == EGL_NO_IMAGE_KHR) {
         qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to record frame: Error creating EGLImageKHR - " << WaylandIntegration::formatGLError(glGetError());
+        pw_stream_queue_buffer(pwStream, buffer);
         return false;
     }
 
@@ -553,7 +581,7 @@ bool ScreenCastStream::recordFrame(gbm_bo *bo, quint32 width, quint32 height, qu
 
     spa_buffer->datas[0].chunk->offset = 0;
     spa_buffer->datas[0].chunk->size = spa_buffer->datas[0].maxsize;
-    spa_buffer->datas[0].chunk->stride = destStride;
+    spa_buffer->datas[0].chunk->stride = streamStride;
 
     pw_stream_queue_buffer(pwStream, buffer);
     return true;
